//
// runEventDisplay.C macro
// By: S. Blusk, J. Buchmann and R. Bleakley
// Summer 2011 Quarknet project
//
// History of updates
//      Sept. 2, 2011  -- Rolled out for testing, S. Blusk
//
// To execute:
// root> .x runEventDisplay.C
//
//   Once you get the first canvas, double-click on the bottom right corner of the middle 
//   canvas to advance to next event
//
// Notes: 
// [1] make sure that in B2DPi.h and B2D3Pi.h that the constructor
// points to the proper location of B2DPi.root and B2D3Pi.root
//
#include <TLatex.h>
#include <TCanvas.h>
#include <TPad.h>
#include <TF1.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TMath.h>
#include <TFile.h>
#include <TStyle.h>
#include <TROOT.h>
#include <TTree.h>
#include <string>
#include "B2DPi.C"
#include "B2D3Pi.C"

// Position and size of XZ canvas
double xz1 = 250.0;
double yz1 = 250.0;
double xzw = 700.0;
double yzw = 450.0;

//  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>
//  <<<<<   Parameters you can change, but probably don't need to >>>>>>
//  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>
int nevUpdate = 20;  // #events between histogram updates
int ShowAllVertices = -1;  // Set to 1 to show all PVs (not only B PV)
double minFlightDistOfB = 10.0;  // minimum flight distance of B for displayed events (mm)
double minFlightDistOfD = 10.0; // minimum flight distance of D for displayed events (mm)
int beampause = 1000000;  // pause between steps of incoming bunches of protons (def = 1000000)
int interactionPause = 1000000;  // pause between updating canvas for particles emerging from interaction def=1000000)
int showAllVerticesPause = 1000000; // pause before showing all PVs (def = 1000000)
//

void setHistProp(TH1F* h, TString xtitle = " ", TString ytitle = " ")
{
  h->SetLineWidth(2);
  h->SetNdivisions(505,"X");
  h->SetNdivisions(505,"Y");
  h->SetXTitle(xtitle);
  h->SetYTitle(ytitle);
  h->GetXaxis()->SetTitleSize(0.06);
  h->GetYaxis()->SetTitleSize(0.06);
  h->GetXaxis()->SetLabelSize(0.05);
  h->GetYaxis()->SetLabelSize(0.05);
  h->GetXaxis()->SetTitleOffset(0.8);
  h->GetYaxis()->SetTitleOffset(0.9);
}

void pause(int n)
{
  for(int i=0;i<n;i++){continue;}
  return;
}

void runEventDisplay()
{  

  // tt.Loop();  

  char input;  // a character variable

  gStyle->SetStatW(0.4);
  gStyle->SetStatH(0.2);
  gStyle->SetTitleSize(0.3);
  gStyle->SetPadLeftMargin(0.13);
  gStyle->SetPadBottomMargin(0.13);  
  gStyle->SetTitleXSize(0.06);	

  int iProc = 1;
  cout << "Which type of decay mode would you like to look at?" << endl;
  cout << " --> For  B -->D pi,        enter 1" << endl;
  cout << " --> For  B -->D pi pi pi,  enter 2" << endl;
  cout << " --> Please answer (1 or 2) : ";
  
  cin >> iProc;
  if(iProc < 1 || iProc>2) break;

  int nAnim;
  cout << "How many event animations do you wish to see [ after this, just fill histograms ]?" << endl;
  cin >> nAnim;
  
  B2DPi ttPi;
  B2D3Pi tt3Pi;
  
  // Book histograms
  TH1F *bmass = new TH1F("bmass","B#rightarrowD#pi Mass",60,5000,5600);
  TH1F* benergy = new TH1F("benergy","B#rightarrowD#pi Energy (GeV)",50,0,500);
  TH1F* bdecaytime = new TH1F("bdecaytime","B#rightarrowD#pi Decay Time (ps)",50,0,10);
  TH1F* bflightdist = new TH1F("bflightdist","B#rightarrowD#pi Flight Distance (mm)",50,0,50);
  if(iProc == 2){
    bmass->SetTitle("B#rightarrowD#pi#pi#pi Mass");
    benergy->SetTitle("B#rightarrowD#pi#pi#pi Energy (GeV)");
    bdecaytime->SetTitle("B#rightarrowD#pi#pi#pi Decay Time (ps)");
    bflightdist->SetTitle("B#rightarrowD#pi#pi#pi Flight Distance (mm)");
  }
  setHistProp(bmass, "Mass (GeV/c^{2})", "Number/10 MeV/c^{2}");
  setHistProp(benergy, "Energy (GeV)", "Number/10 GeV");
  setHistProp(bdecaytime, "Decay Time (ps)", "Number/0.2 ps");
  setHistProp(bflightdist, "Distance (mm)", "Number/mm");  


  // Number of Entries
  Long64_t nentries = ttPi.fChain->GetEntriesFast();
  if(iProc==2) nentries = tt3Pi.fChain->GetEntriesFast();

  // Create Canvases
  TCanvas *cp = new TCanvas("cp","Distributions",970,0,400,400);
  cp->SetFillColor(kYellow-9);
  cp->Divide(2,2);

  // Loop over events
  int j = 0;
  for(int i=0; i<nentries;i++){
      if(iProc==1){        
        Long64_t ientry = ttPi.LoadTree(i);
        if (ientry < 0) break;
        ttPi.fChain->GetEntry(i);
        ttPi.LoadToLocalVariables();
      }else if(iProc==2){
        Long64_t ientry = tt3Pi.LoadTree(i);
        if (ientry < 0) break;
        tt3Pi.fChain->GetEntry(i);
        tt3Pi.LoadToLocalVariables();
      }
      
      double dzb = BVERTEX_z - BPV_z;
      double dzd = DVERTEX_z - BVERTEX_z;
    
      // Use flight distance cuts only for the first few events that we're animating
      if(j < nAnim){
        if(dzb<minFlightDistOfB) continue;
        if(dzd<minFlightDistOfD) continue;
      }
      
	  TCanvas *ccxy;
	  TCanvas *ccxz;

	  if(j==0){
		  ccxz = BlankCanvas();
		  cout << endl;
		  cout << " << Double click on lower right edge of the canvas to proceed >> " << endl;
		  ccxz->WaitPrimitive();
	  }

      if(B_MCMatch >=30) continue;
      if(j < nAnim){
        // Create Event Display
	    ccxz = drawPictureXZ_Anim(iProc);
		ccxy = drawPictureXY(iProc);
        ccxz->Update();
        ccxy->Update();
      }
      
      // Fill histograms
      bmass->Fill(B_Mass);
      benergy->Fill(B_Energy);
      bdecaytime->Fill(B_DecayTime);
      bflightdist->Fill(B_FlightDist);

      // Draw Histograms
      cp->cd(1)->SetFillColor(kGreen-9); bmass->DrawCopy();
      cp->cd(2)->SetFillColor(kGreen-9); benergy->DrawCopy();
      cp->cd(3)->SetFillColor(kGreen-9); bdecaytime->DrawCopy();
      cp->cd(4)->SetFillColor(kGreen-9); bflightdist->DrawCopy();
      // Only update every N events (N = nevUpdate value), OR for the first N animated events
      if(j%nevUpdate == 0 || j<nAnim) cp->Update();   
     
      if(j < nAnim){
	      ccxy->Update();
	      //ccxy->RaiseWindow();
          //ccxy->Draw();
	      printf("\n <<< Double click in the bottom right corner of the pad to continue >>> \n");
	      ccxz->WaitPrimitive();
		  //cout << " Continue to next event (y or n)";
        //cin >> input;
        //if (input == 'n' || input=='N') break;
        if(j < (nAnim-1) ) ccxy->Clear(); // clear up to the (n-1)th display
	  }
      j++;
	  pause(100000);
  }
}

TCanvas* BlankCanvas()
{
  double zl = BPV_z + zneg;
  double zh = DVERTEX_z + zpos;
  double xl = (BVERTEX_x + BPV_x)/2.0 - xscale;
  double xh = (BVERTEX_x + BPV_x)/2.0 + xscale;  
  TCanvas *c =  new TCanvas("c", "", xz1, yz1, xzw, yzw); 
  c->Range(zl, xl, zh, xh);
  c->SetFillColor(kBlack);
  c->Update();
  c->Draw();
  c->Update();
  return c;
}

TCanvas* drawPictureXZ_Anim(int iProc)
{
  TCanvas *c =  new TCanvas("c", "", xz1, yz1, xzw, yzw);  

  double zl = BPV_z + zneg;
  double zh = DVERTEX_z + zpos;
  double xl = (BVERTEX_x + BPV_x)/2.0 - xscale;
  double xh = (BVERTEX_x + BPV_x)/2.0 + xscale;      

  c->Range(zl, xl, zh, xh);
  c->SetFillColor(kBlack);
  c->Update();
  c->Draw();

  double zEnd = 250.0;
  cout << " ===================================================" << endl;
  cout << "Number of Primary Vertices = " << NumPV << endl;

  //----------------------------------------------
  // Animate the protons coming into the B vertex
  //----------------------------------------------
  double xsiz = 0.15;
  int nstepb = 20;
  
  static const int nproton = 14;
  double zsiz = xsiz*(zh-zl)/(xh-xl);
  TEllipse* e1[nproton];
  TEllipse* e2[nproton];
  for(int j=0; j<nproton; j++){
    e1[j] = new TEllipse(0, 0, zsiz, xsiz, 0, 360,0);          
    e2[j] = new TEllipse(0, 0, zsiz, xsiz, 0, 360,0);          
  }

  // Draw "star" as sort of explosion
  TMarker *m = new TMarker(BPV_z, BPV_x, 29);
  
  double pposz[nproton]={0,10,20,30,5, 15, 25, 5,15,25,10,20,10,20};
  double pposx[nproton]={0,0,0,0,0.25,0.25,0.25,-0.25,-0.25,-0.25,0.5,0.5,-0.5,-0.5};

  double zstepl = abs(zl-BPV_z)/nstepb;
  double zstepr = abs(zh-BPV_z)/nstepb;      

  // Get closest point between left and right beams
  double mindiff = 1000;
  for(int i=0; i<2*nstepb; i++){
    double poszl0 = zl+i*zstepl;    
    double poszr0 = zh-i*zstepr;
    double diff = fabs(poszl0-poszr0);
    if(diff < mindiff) mindiff = diff;
  }
  
  for(int i=0; i<4*nstepb; i++){
    double poszl0 = zl+i*zstepl;    
    double poszr0 = zh-i*zstepr;
    for(int j=0; j<nproton; j++){
      double poszl = poszl0 + pposz[j];
      double poszr = poszr0 + pposz[j];
      double posxl = pposx[j];
      double posxr = pposx[j];
      e1[j]->SetFillStyle(1001);
      e1[j]->SetFillColor(kWhite);
      e2[j]->SetFillStyle(1001);
      e2[j]->SetFillColor(kWhite);
      e1[j]->SetX1(poszl);
      e1[j]->SetY1(posxl);
      e2[j]->SetX1(poszr);
      e2[j]->SetY1(posxr);
      e1[j]->Draw();
      e2[j]->Draw();      
    }
   
    if(fabs(poszl0-poszr0) < mindiff + 1){
      for(int k=0; k<6; k++){
        m->SetMarkerSize(10*k);    
        m->SetMarkerColor(kRed);
        if(k%2==0) m->SetMarkerColor(kGreen);
        m->Draw();
        c->Update();
        pause(beampause);
      }
    }
    m->SetMarkerSize(0);
    c->Update();
    if(poszl0 > zh && poszr0 < zl) break;
    
  }
  // Now make them go off screen
  for(int j=0; j<nproton; j++){
    e1[j]->SetX1(-10000);e2[j]->SetX1(10000); 
    e1[j]->Draw(); e2[j]->Draw();
  }
  c->Update();    
    
  //----------------------------------------------------
  // Now animate the tracks coming out of the B vertex
  //----------------------------------------------------

  TLine* el[200];
  TLine* lb;
  TLine* ld;
  TLine* lbdau[3];
  TLine* lddau[3];

  for(int i=0; i<200; i++){
    el[i] = new TLine(0,0,10,10);
  }
  lb = new TLine(0,0,10,10);  
  ld = new TLine(0,0,10,10);  
  for(int i=0; i<3; i++){
    lbdau[i] = new TLine(0,0,10,10);  
    lddau[i] = new TLine(0,0,10,10);  
  }
  

  
  int nstep = 40;
  double zstepl = abs(zl-BPV_z)/nstep;
  double zstepr = abs(zh-BPV_z)/nstep;      
  int iTrk = nTracksBPV;
  for(int j=0; j<nstep+10; j++){
    for(int i=0; i<iTrk; i++){
        double xsl = xslopeBPV[i];
        double ysl = yslopeBPV[i];
        double xint = xintcBPV[i];
        double yint = yintcBPV[i];
        double zf = zFinalBPV[i];
		double zstep = zstepr;
	    if(zf<0) zstep = (zf-BPV_z)/nstep;
        double zz = BPV_z + j*zstep;
        double posz = zz;
        double posx = BPV_x + xsl*(zz - BPV_z);
        double z1 = BPV_z;
        double z2 = zz+zstepr/2.;
        double x1 = BPV_x + xsl*(z1 - BPV_z);
        double x2 = BPV_x + xsl*(z2 - BPV_z);
        el[i]->SetX1(z1);el[i]->SetY1(x1);el[i]->SetX2(z2);el[i]->SetY2(x2);
        el[i]->SetLineColor(14);
        el[i]->Draw();
	  }    
    // Now animate the B particle track
    double zz = BPV_z + j*zstepr;
    double posz = zz;
    double posx = BPV_x + xslopeB*(zz - BPV_z);
    double r = sqrt((posx-BPV_x)*(posx-BPV_x)+(posz-BPV_z)*(posz-BPV_z));
    double rb = sqrt((BVERTEX_x-BPV_x)*(BVERTEX_x-BPV_x)+(BVERTEX_z-BPV_z)*(BVERTEX_z-BPV_z));
    double rd = sqrt((DVERTEX_x-BPV_x)*(DVERTEX_x-BPV_x)+(DVERTEX_z-BPV_z)*(DVERTEX_z-BPV_z));
    if(r < rb){
      //double zstepl = (zh - BPV_z)/nstep;
      double zz = BPV_z + j*zstepr;
      double z1 = BPV_z;
      double z2 = zz+zstepr/2.;
      if(z2 > BVERTEX_z) z2 = BVERTEX_z;
      double x1 = BPV_x + xslopeB*(z1 - BPV_z);
      double x2 = BPV_x + xslopeB*(z2 - BPV_z);
      lb->SetX1(z1);lb->SetY1(x1);lb->SetX2(z2);lb->SetY2(x2);      
      lb->SetLineColor(kViolet);
      lb->SetLineWidth(2);
      lb->SetLineStyle(7);  
      lb->Draw();      
    }
    // If radius larger than B vertex radius, and less then D vertex radius, animate the D
    if(r>rb && r<rd){
      //double zstepl = (zh - BPV_z)/nstep;
      double zz = BPV_z + j*zstepr;
      double z1 = BVERTEX_z;
      double z2 = zz+zstepr/2.;
      if(z2 > DVERTEX_z) z2 = DVERTEX_z;
      if( (z2 + zstepr/2.) >  DVERTEX_z) z2 = DVERTEX_z;
      double x1 = BVERTEX_x + xslopeD*(z1 - BVERTEX_z);
      double x2 = BVERTEX_x + xslopeD*(z2 - BVERTEX_z);
      ld->SetX1(z1);ld->SetY1(x1);ld->SetX2(z2);ld->SetY2(x2);      
      ld = new TLine(z1,x1,z2,x2);
      ld->SetLineColor(kOrange);
      ld->SetLineWidth(2);  
      ld->SetLineStyle(2);  
      ld->Draw();
    }   
   
    // Draw bachelor tracks associated with the B vertex
    if(r>rb){
	  int nBachTracks = 1;
	  if(iProc==2) nBachTracks = 3;
      for(int k=0; k<nBachTracks; k++){
        //double zstepl = (zh - BPV_z)/nstep;
        double zz = BPV_z + j*zstepr;
        double z1 = BVERTEX_z;
        double z2 = zz+zstepr/2.;
        double x1 = BVERTEX_x + xslopeBdau[k]*(z1 - BVERTEX_z);
        double x2 = BVERTEX_x + xslopeBdau[k]*(z2 - BVERTEX_z);
        lbdau[k]->SetX1(z1);lbdau[k]->SetY1(x1);lbdau[k]->SetX2(z2);lbdau[k]->SetY2(x2);
        lbdau[k]->SetLineColor(kBlue);
        lbdau[k]->SetLineWidth(1);
        lbdau[k]->Draw();
      }
    }
    // Draw D daughter tracks associated with the D vertex
    if(r>rd){
      for(int k=0; k<3; k++){
        //double zstepl = (zh - BPV_z)/nstep;
        double zz = BPV_z + j*zstepr;
        double z1 = DVERTEX_z;
        double z2 = zz+zstepr/2.;
        double x1 = DVERTEX_x + xslopeDdau[k]*(z1 - DVERTEX_z);
        double x2 = DVERTEX_x + xslopeDdau[k]*(z2 - DVERTEX_z);
        lddau[k]->SetX1(z1);lddau[k]->SetY1(x1);lddau[k]->SetX2(z2);lddau[k]->SetY2(x2);
        lddau[k]->SetLineColor(kRed);
        lddau[k]->SetLineWidth(1);
        lddau[k]->Draw();
	  }
    }    
    pause(interactionPause);    
    c->Update();
  }


  if(ShowAllVertices > 0){
    pause(showAllVerticesPause);
    // Now add additional PVs
    for(int j=0; j<NumPV; j++){
      int iTrk = nPVTracks[j];
      if(fabs(PV_z[j]-BPV_z)<0.01) continue;
      
      for(int i=0; i<iTrk; i++){    
        double xsl = xslopePV[i][j];
        double ysl = yslopePV[i][j];
        double xint = xintcPV[i][j];
        double yint = yintcPV[i][j];
        double zf = zFinalPV[i][j];
        double xf = xsl*zf + xint;
        double yf = ysl*zf + yint;        
        l = new TLine(PV_z[j],xint, zf, xf);     
        l->SetLineColor(12);
        l->SetLineWidth(1);
        l->Draw();      
      }    
      c->Update();
    }
    
  }
    
    
  double dx = 0.05*(xh-xl);
  double dx1 = 0.035*(xh-xl);
  double dx2 = 0.07*(xh-xl);
  
  TLatex t;
  t.SetTextAlign(22);
  t.SetTextSize(0.04);
  t.SetTextColor(kBlue);
  t.DrawLatex(BVERTEX_z+0.5,BVERTEX_x+dx1 ,"B_{vertex}");
  t.SetTextColor(kRed);
  t.DrawLatex(DVERTEX_z+ 0.2 , DVERTEX_x+dx2 , "D_{vertex}");

  //TLatex t;
  t.SetTextSize(0.08); 
  t.SetTextAlign(12);
  t.SetTextColor(kRed);
  t.DrawLatex(zl+15,xl+dx,"Event Display -- LHCb");

  double z1 = BPV_z-5;
  double z2 = z1 + 10;
  double zz = (z1 + z2)/2.;
  double xx1 = xl+3.5*dx;
  double xx2 = xx1 - dx;
  TLine *lz = new TLine(z1,xx1,z2,xx1);
  lz->SetLineColor(kWhite);
  lz->SetLineWidth(2);
  lz->Draw();
  t.SetTextSize(0.03); 
  t.SetTextColor(kWhite);
  t.SetTextAlign(22);
  t.DrawLatex((z1+z2)/2.,xx2,"1 cm");
  double xxa = xx1 + 1.0;
  double xxb = xx1 - 1.0;
  TLine *lx = new TLine(zz,xxa,zz,xxb);
  lx->SetLineColor(kWhite);
  lx->SetLineWidth(2);
  lx->Draw();
  t.SetTextSize(0.03); 
  t.SetTextColor(kWhite);
  t.SetTextAlign(22);
  t.SetTextAngle(90);
  t.DrawLatex(z2+5.0,xx1,"2 mm");
  

  c->Update();
  c->Draw();

  return c;

}

TCanvas* drawPictureXY(int iProc)
{
  TCanvas *cc =  new TCanvas("cc", "", 0, 0, 300, 300);  

  double xl = (BPV_x+DVERTEX_x)/2 - xscaleXY;
  double xh = xl + 2*xscaleXY;
  double yl = (BPV_y + DVERTEX_y)/2.0 - xscaleXY;
  double yh = yl+2*xscaleXY;
  cc->Range(xl, yl, xh, yh);
  cc->SetFillColor(kBlack);
  
  cout << "  B Prim. Vertex Pos (x,y,z): " << BPV_x << " " << BPV_y << " " << BPV_z << endl;
  cout << "  B Vertex Pos (x,y,z):       " << BVERTEX_x << " " << BVERTEX_y << " " << BVERTEX_z << endl;
  cout << "  D Vertex Pos (x,y,z):       " << DVERTEX_x << " " << DVERTEX_y << " " << DVERTEX_z << endl;
  //cout << " D daughters: " << DDAU_Px[0] << " " << DDAU_Py[0] << " " << DDAU_Pz[0] << endl;
  //cout << " D daughters: " << DDAU_Px[1] << " " << DDAU_Py[1] << " " << DDAU_Pz[1] << endl;
  //cout << " D daughters: " << DDAU_Px[2] << " " << DDAU_Py[2] << " " << DDAU_Pz[2] << endl;
  double xEnd = 20.0;  
  if(B_Px<0) xEnd = -20.0;
  

  TLine * l; 

  cout << "Number of proton-proton Collsions = " << NumPV << endl;
  
  TArrow *a;
  for(int j=0; j<NumPV; j++){
    int iTrk = nPVTracks[j];
    bool bVertex = false;
    if(fabs(PV_z[j]-BPV_z)<0.01) bVertex = true;
    if(!bVertex) cout << " ---> pp Collision # " << j << " has " << iTrk << " tracks" << endl;
    if(bVertex) cout << " ---> pp Collision # " << j << " has " << iTrk << " tracks (B Primary Vertex)" << endl;
    if(!bVertex) continue;    
    
    for(int i=0; i<iTrk; i++){    
      double xsl = 6.28*(gRandom->Rndm()-0.5);
      double ysl = 6.28*(gRandom->Rndm()-0.5);
      xsl = tan(xsl);
      ysl = tan(ysl);      
      double xint = 0.03*gRandom->Gaus();
      double yint = 0.03*gRandom->Gaus();
      double xint = PV_x[j]+xint;      
      double yint = PV_y[j]+yint;      
      double r = gRandom->Rndm();
      double zf = xEnd;
      if(r<0) zf = -xEnd;
      double xf = xsl*zf + xint;
      double yf = ysl*zf + yint;

      l = new TLine(xint, yint, xf, yf);     
      l->SetLineColor(12);
      l->SetLineWidth(1);
      if(bVertex) l->SetLineColor(14);
      l->Draw();
    }
    cc->Update();
    if(fabs(PV_z[j]-BPV_z)<0.01) bVertex = true;
    
  }
  //Draw B particle track

  l = new TLine(BPV_x,BPV_y,BVERTEX_x,BVERTEX_y);
  l->SetLineStyle(7);
  l->SetLineWidth(3);
  l->SetLineColor(kViolet);  
  l->Draw();
  cc->Update();

  
  // Draw line for D trajectory
  l = new TLine(BVERTEX_x,BVERTEX_y,DVERTEX_x,DVERTEX_y);
  l->SetLineStyle(7); l->SetLineWidth(3); l->SetLineColor(kOrange);  
  l->Draw();
  cc->Update();

  int nBachTracks = 1;
  if(iProc==2) nBachTracks = 3;
  // Draw daughters coming from B vertex
  for(int i=0; i<nBachTracks; i++){
     double ysl = BDAU_Py[i]/BDAU_Px[i];
	 double yint = BDAU_POSy[i]-ysl*BDAU_POSx[i];
	 double xEnd2 = 10.0;  
     if(BDAU_Px[i]<0) xEnd2 = -10.0;
     double yf = ysl*xEnd2+yint;
     l = new TLine(BVERTEX_x, BVERTEX_y, xEnd2, yf); 
     l->SetLineColor(kBlue); l->SetLineWidth(2);
     l->Draw();    
  }
  cc->Update();
  
  // Draw D daughters
  for(int i=0; i<3; i++){ 
    double ysl = DDAU_Py[i]/DDAU_Px[i];
    double yint = DDAU_POSy[i]-ysl*DDAU_POSx[i];
    double xEnd2 = 10.0;  
    if(DDAU_Px[i]<0) xEnd2 = -10.0;
    double yf = ysl*xEnd2+yint;
    l = new TLine(DVERTEX_x, DVERTEX_y, xEnd2, yf); 
    l->SetLineColor(kRed); l->SetLineWidth(2);
    l->Draw();    
  }

  TLatex t;
  t.SetTextSize(0.06); 
  t.SetTextAlign(12);
  t.SetTextColor(kGreen);
  t.DrawLatex(xl+0.5,yl+0.4,"Event Display -- LHCb (XY View)");
  
  cc->Update();
  return cc;

}


